#include <qapplication.h>
#include <qpixmap.h>
#include <qimage.h>

// Fill a rectangle in a QImage with the given color.
static void fillImageRect( QImage* img,
			   int x,
			   int y,
			   int w,
			   int h,
			   const QColor& col )
{
    if ( x < 0 ) x = 0;
    if ( y < 0 ) y = 0;
    if ( x + w > img->width() ) w = img->width() - x;
    if ( y + h > img->height() ) h = img->height() - y;
    if ( w < 0 || h < 0 ) return;
    if (img->depth() != 32)
    {
	qWarning( "fillImageRect: need 32 bpp to do alpha-blending!" );
	return;
    }

    int rgb = col.rgb();
    for (int i = y; i < y + h; ++i)
    {
        // inspired by QImage::fill
        uint* p = (uint*) img->scanLine(i) + x;
        uint *end = p + w;
        while ( p < end )
            *p++ = rgb;
    }
}

// Copy a portion of src onto dst without doing alpha blending
// This assumes both images are 32-bit.
static void copyImageRect( QImage* dst, int x, int y, const QImage* src,
                           int sx, int sy, int sw, int sh )
{
    bool has_alpha = src->hasAlphaBuffer();
    if ( has_alpha ) {
	// alpha channel should be only copied, not used by bitBlt()
	const_cast<QImage *>(src)->setAlphaBuffer( FALSE );
    }
    bitBlt( dst, x, y, src, sx, sy, sw, sh );
    if ( has_alpha ) {
	// restore image state
        const_cast<QImage *>(src)->setAlphaBuffer( TRUE );
    }
}

class Viewer :public QWidget
{
public:
  Viewer( QWidget* parent ) : QWidget( parent )
  {
    QImage bgImg("background.png"); // 8 bit
    bgImg = bgImg.convertDepth( 32 ); // -> 32 bit
    // Create the image containing fixed colors with transparency
    QImage img( bgImg.size(), 32 );
    img.fill( 0 ); // do this first!
    img.setAlphaBuffer( true );
    int alphaValue = (int)(256 - 0.6 * 256); // transparency level 0.6
    int x = 20;
    int w = (bgImg.width() - 2*x)/4; // width of each band
    int y = 40;
    QColor col = qRgba( 255, 0, 0, alphaValue); // transparent red
    fillImageRect( &img, x, y, w, bgImg.height() - 2*y, col );
    col = qRgba( 0, 255, 0, alphaValue ); // transparent green
    x += w;
    fillImageRect( &img, x, y, w, bgImg.height() - 2*y, col );
    col = qRgba( 0, 0, 255, alphaValue ); // transparent blue
    x += w;
    fillImageRect( &img, x, y, w, bgImg.height() - 2*y, col );

    // Alpha-blending
    bitBlt( &bgImg, 0, 0, &img );

    // Example using copyImageRect - i.e. no alpha blending
    y += (bgImg.height()-80)/3;
    copyImageRect( &bgImg, 20, y,
                   &img, 20, y, bgImg.width()-40, (bgImg.height()-80)/3 );

    // Convert back to a pixmap
    // no need for the alpha channel in the pixmap
    bgImg.setAlphaBuffer( false );
    buffer.convertFromImage( bgImg );

    setFixedSize( buffer.size() );
  }

  virtual void paintEvent ( QPaintEvent * )
  {
    bitBlt( this, 0, 0, &buffer );
  }

private:
    QPixmap buffer;
};

int main( int argc, char** argv )
{
  QApplication app( argc, argv );

  Viewer* viewer = new Viewer(0);
  app.setMainWidget( viewer );
  viewer->show();

  return app.exec();
}
