#include <qapplication.h>
#include <qmainwindow.h>
#include <qmenubar.h>
#include <qpainter.h>
#include <qpaintdevicemetrics.h>
#include <qprinter.h>

class DrawArea : public QWidget
{
    Q_OBJECT
public:

    DrawArea( QWidget* parent, const char* name = 0 );

public slots:
    void slotPrint();

protected:

    virtual void paintEvent( QPaintEvent* ) {
        QPainter painter( this );
        paintTo( painter );
    }

    void paintTo( QPainter& painter );

private:
    QPrinter printer;
};

DrawArea::DrawArea( QWidget* parent, const char* name )
    :QWidget( parent, name )
{
    QMenuBar* menubar = new QMenuBar( this, "menu" );
    Q_CHECK_PTR( menubar );
    menubar->insertItem( "&Print",         this, SLOT(slotPrint()       ) );
}

void DrawArea::slotPrint()
{
    if ( printer.setup(this) ) {
        qDebug( "\nPrinting the window..." );
        QPainter p;
        if( !p.begin( &printer ) )
            return;
        paintTo( p );
        p.end();
        qDebug( "Printing completed" );
    } else {
        qDebug( "Printing aborted" );
    }
}

void DrawArea::paintTo( QPainter& p )
{
    // find the margins between printable area and paper edges
    uint top, left, bottom, right;
    if( &printer == p.device() )
        printer.margins( &top, &left, &bottom, &right );
    else{
        top=0;
        left=0;
        bottom=0;
        right=0;
    }

    // find out how many points is 1 cm in both directions
    QPaintDeviceMetrics metrics( p.device() );
    const int metHeight = metrics.height();
    const int metWidth = metrics.width();
    const int dpiX = metrics.logicalDpiX();
    const int dpiY = metrics.logicalDpiY();
    const double cmX = 1.0/2.54*dpiX; // 1 cm width in log. coordinates
    const double cmY = 1.0/2.54*dpiY; // 1 cm height in log. coordinates

    // mark the upper/left and lower/right corner
    p.setPen( QPen( Qt::red, 1 ) );
    p.drawLine(0, 0, static_cast <int> (5.0*cmX), 0);
    p.drawLine(0, 0, 0, static_cast <int> (5.0*cmY));
    p.drawLine(metWidth-1, metHeight-1,
               metWidth-1-static_cast <int> (5.0*cmX), metHeight-1);
    p.drawLine(metWidth-1, metHeight-1,
               metWidth-1, metHeight-1-static_cast <int> (5.0*cmY));

    p.drawText(static_cast <int> (cmX), static_cast <int> (cmY),
               "Paper margins for current drawing device:");
    p.drawText(static_cast <int> (cmX), static_cast <int> (2.0*cmY),
               QString("top: %1  left: %2  bottom: %3  right: %4")
               .arg(top    / cmY)
               .arg(left   / cmX)
               .arg(bottom / cmY)
               .arg(right  / cmX));

    // draw a 8 cm x 8 cm rectangle at a physical point (4 cm, 4 cm)
    QRect body( static_cast <int> (4.0*cmX)-left,
		static_cast <int> (4.0*cmY)-top,
                static_cast <int> (8.0*cmX),
		static_cast <int> (8.0*cmY) );
    p.setPen( QPen( Qt::blue, 1 ) );
    p.drawRect( body );
    p.setPen( QPen( Qt::black, 1 ) );
    p.drawText(static_cast <int> (cmX), static_cast <int> (13.0*cmY)-top,
               "This rectangle has a size of 8 cm x 8 cm");
    p.drawText(static_cast <int> (cmX), static_cast <int> (14.0*cmY)-top,
               "and is 4 cm away from upper left / top edges.");
}

int main( int argc, char** argv ) {
    QApplication app( argc, argv );

    DrawArea* test = new DrawArea( 0 );
    test->show();

    QObject::connect( qApp, SIGNAL( lastWindowClosed() ),
                      qApp, SLOT( quit() ) );
    return app.exec();
}

#include "main.moc"
